/*
 * DBConnector
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * dbConnection.c
 *
 * compile with:
 * gcc -I <install_dir>/include/ dbConnection.c <install_dir>/lib/libdb.a -lpq
 *
 *  Created on: June 07, 2010
 *      Author: Timo Minartz
 */

#include "DBConnector.h"
#include <unistd.h> // for sleep
#include <assert.h> // for asserts

int main(int argc, char** argv) {

	// if not on eeClust-cluster, use the following line to connect to the database
	//	initConnectionWithArgs(1, "host = localhost dbname = watts user = testuser password = test");
	db_initConnection(1);

	const char * node = "testnode";

	createNodeMetric("power", node);

	uint64_t microseconds = 10;
	double power1 = 100;
	double power2 = 200;

	int ret = db_pushNodeMetric("power", node, microseconds, power1);

	if (ret < 0) {
		if (ret == DB_CONNECTION_FAILED)
			printf("Connection to database failed.\n");
		if (ret == DB_QUERY_FAILED)
			printf("Query failed.\n");

		return ret;
	}

	ret = db_pushNodeMetric("power", node, microseconds + 10, power2);

	if(ret < 0)
		printf("pushNodeData failed with error code %d\n", ret);

	ret = db_pushNodeMetric("power", node, microseconds + 20, power2);

	if(ret < 0)
		printf("pushNodeData failed with error code %d\n", ret);

	ret = db_pushNodeMetric("power", node, microseconds + 30, power1);

	if(ret < 0)
		printf("pushNodeData failed with error code %d\n", ret);

	ret = db_pushNodeMetric("power", node, microseconds + 40, power2);

	if(ret < 0)
		printf("pushNodeData failed with error code %d\n", ret);

	ret = db_pushNodeMetric("power", node, microseconds + 50, power1);

	if(ret < 0)
		printf("pushNodeData failed with error code %d\n", ret);

	ret = db_pushNodeMetric("power", node, microseconds + 60, power2);

	if(ret < 0)
		printf("pushNodeData failed with error code %d\n", ret);

	ret = db_pushNodeMetric("power", node, microseconds + 70, power1);

	if(ret < 0)
		printf("pushNodeData failed with error code %d\n", ret);

	// wait for query puffer
	sleep(1);

	row_struct *rows;

	int crows = db_getNodeMetric("power", node, microseconds, microseconds + 70, &rows);

	if (crows >= 0) {
		int i = 0;
		for (i = 0; i < crows; ++i) {
			printf("Timestamp: %lu : %lf W\n", rows[i].timestamp, rows[i].value);
		}

		if( i == 0) {
			printf("No entries for node %s\n", node);
		} else {
			free(rows);
		}
	} else {
		printf("getNodeData failed with error code %d\n", ret);
	}

	char ** nodes;

	int cnodes = getNodes(&nodes);

	if (cnodes >= 0) {

		printf("Found %d node(s).\n", cnodes);

		if (cnodes > 0) {
			int i;
			for (i = 0; i < cnodes; ++i) {
				printf("Node: %s\n", nodes[i]);
				free(nodes[i]);
			}

			free(nodes);
		}
	} else {
		printf("getNodes failed with error code %d\n", cnodes);
	}

	const char * metric = "cpu_0_cstate_0";
	uint64_t microseconds1 = 10;
	uint64_t microseconds2 = 30;
	double cstate1 = 100;
	double cstate2 = 200;

	createNodeMetric(metric, node);
	ret = db_pushNodeMetric(metric,node, microseconds1, cstate1);

	if (ret < 0) {
		if (ret == DB_CONNECTION_FAILED)
			printf("Connection to database failed.\n");
		if (ret == DB_QUERY_FAILED)
			printf("Query failed.\n");

		return ret;
	}

	ret = db_pushNodeMetric(metric,node, microseconds2, cstate2);

	if(ret < 0)
		printf("pushNodeMetric failed with error code %d\n", ret);

	// wait for query puffer
	sleep(1);

	crows = db_getNodeMetric(metric, node, microseconds1, microseconds2, &rows);

	if (crows > 0) {
		int i = 0;
		for (i = 0; i < crows; ++i) {
			printf("Timestamp: %lu : %lf s\n", rows[i].timestamp, rows[i].value);
		}

		free(rows);
	} else {
		printf("getNodeMetric failed with error code %d\n", ret);
	}

	cnodes = getNodes(&nodes);

	if (cnodes > 0) {

		printf("Found %d node(s).\n", cnodes);

		if (cnodes > 0) {
			int i;
			for (i = 0; i < cnodes; ++i) {
				printf("Node: %s\n", nodes[i]);
				free(nodes[i]);
			}

			free(nodes);
		}
	} else {
		printf("getNodes failed with error code %d\n", cnodes);
	}

	char ** metrics = NULL;

	int cmetrics = getMetrics(node,&metrics);

	if (cmetrics >= 0) {

		printf("Found %d metrics(s) for node %s.\n", cmetrics,node);

		if (cmetrics > 0) {
			int i;
			for (i = 0; i < cmetrics; ++i) {
				printf("Metric: %s\n", metrics[i]);
				free(metrics[i]);
			}

			free(metrics);
		}
	} else {
		printf("getMetrics failed with error code %d\n", cmetrics);
	}

	ret = deleteNodeTable(node);

	if(ret < 0)
			printf("deleteNodeTable failed with error code %d\n", ret);

	ret = deleteMetric(metric,node);

	if(ret < 0)
			printf("deleteMetric failed with error code %d\n", ret);

}
