/*
 * DBConnector
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * MetricInterpreter.h
 *
 *  Created on: Oct 7, 2010
 *      Author: Timo Minartz
 */

#ifndef METRICINTERPRETER_H_
#define METRICINTERPRETER_H_

#include "DBConnector.h"

/**
 * @param microseconds time in microseconds
 * @return time in sec (microseconds / 1000000.0)
 */
double mi_getTimestampInSec(uint64_t microseconds);

/**
 * @param energy total energy in Joule
 * @param microseconds time in microseconds
 * @return mean power consumption in watt (energy / mi_getTimestampInSec(microseconds));
 */
double mi_getMeanPowerConsumptionFromEnergy(double energy, uint64_t microseconds);

/**
 * @param power mean power consumption for the measurement duration
 * @param microseconds measurement duration in microseconds
 * @return energy consumption in Joule (power * getTimestampInSec(microseconds))
 */
double mi_getEnergyConsumptionFromMeanPower(double power, uint64_t microseconds);

/**
 * Sums up each power consumption value and divides the sum by the count of rows.
 *
 * @return mean power consumption in watt
 */
double mi_getMeanPowerConsumption(row_struct ** p_nodeRow, int crows);

/**
 * calculates step based mean power consumption based on mi_getStepBasedEnergyConsumption(p_nodeRow, crows)
 * @return mean power consumption in watt
 */
double mi_getStepBasedMeanPowerConsumption(row_struct ** p_nodeRow, int crows);

/**
 * getEnergyConsumptionFromMeanPower(getMeanPowerConsumption(p_nodeRow, crows), microseconds);
 *
 * If have the choise, use mi_getStepBasedEnergyConsumption which has a higher accuracy
 *
 * @return energy consumption in Joule
 */
double mi_getEnergyConsumption(row_struct ** p_nodeRow, int crows);

/**
 * calculates the sum of the energy consumption for every two sequenced timestamps,
 * because the timesteps must not have the same size.
 * @return total energy consumption in Joule
 */
double mi_getStepBasedEnergyConsumption(row_struct ** p_nodeRow, int crows);

/**
 * calculates the sum of the energy consumption for every two sequenced timestamps,
 * because the timesteps must not have the same size.
 * @return total energy consumption in Millijoule
 */
double mi_getStepBasedEnergyConsumptionInMilliJoule(row_struct ** p_nodeRow, int crows);


#endif /* METRICINTERPRETER_H_ */
