/*
 * DBConnector
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * DBConnector.c
 *
 *  Created on: Jan 11, 2011
 *  Author: Timo Minartz
 */
#include <sys/time.h>

#include "DBConnector.h"
#include "QueryPuffer.h"

static QueryPuffer * queryPuffer = NULL;

uint64_t getSystemTimestamp(void) {

	struct timeval detail_time;
	gettimeofday(&detail_time,NULL);

	uint64_t microseconds = detail_time.tv_sec * 1000000 + detail_time.tv_usec;

	return microseconds;
}

int db_initConnection(int bufferSize) {

#ifdef DEBUG
	g_print("DEBUG enabled.\n");
#endif

#ifdef HAVE_PG
	const char * conninfo = "host = master dbname = PowerTracer user = eeclust password = eeclust";
#endif
#ifdef HAVE_SQLITE
	const char * conninfo = "PowerTracer";
#endif

	if(queryPuffer == NULL)
		queryPuffer = QueryPuffer_alloc(bufferSize, conninfo);

	if(queryPuffer == NULL)
		return DB_CONNECTION_FAILED;

#ifdef DEBUG
	g_print("Initialized connection.\n");
#endif

	return OK;
}

int db_pushNodeMetric(const char * metric, const char * node,
		uint64_t timestamp, double value) {

	QueryPuffer_addQuery(queryPuffer, metric,node,timestamp,value);

	return OK;
}

void db_finishConnection(void) {
	QueryPuffer_unref(queryPuffer);
}
