/*
 * DBConnector
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * MetricInterpeter.c
 *
 *  Created on: Oct 7, 2010
 *      Author: Timo Minartz
 */

#include "MetricInterpreter.h"

double mi_getTimestampInSec(uint64_t microseconds) {
	return microseconds / 1000000.0;
}

double mi_getMeanPowerConsumptionFromEnergy(double energy, uint64_t microseconds) {
	return energy / mi_getTimestampInSec(microseconds);
}

double mi_getEnergyConsumptionFromMeanPower(double power, uint64_t microseconds) {
	return power * mi_getTimestampInSec(microseconds);
}

double mi_getMeanPowerConsumption(row_struct ** p_nodeRow, int crows) {
	row_struct * nodeRow = *p_nodeRow;

	// calculate mean power consumption

	double sum = 0.0;
	int j;
	for (j = 0; j < crows; ++j) {
		sum += nodeRow[j].value;
	}

	return sum / crows;
}

double mi_getStepBasedMeanPowerConsumption(row_struct ** p_nodeRow, int crows) {
	row_struct * nodeRow = *p_nodeRow;

	// calculate mean power consumption

	double energy = mi_getStepBasedEnergyConsumption(p_nodeRow, crows);

	if(crows > 0) {
		uint64_t microseconds = nodeRow[crows - 1].timestamp - nodeRow[0].timestamp;
		return mi_getMeanPowerConsumptionFromEnergy(energy,microseconds);
	}

	return 0.0;
}

double mi_getEnergyConsumption(row_struct ** p_nodeRow, int crows) {
	row_struct * nodeRow = *p_nodeRow;

	// calculate energy consumption

	uint64_t microseconds = nodeRow[crows - 1].timestamp
			- nodeRow[0].timestamp;

	return mi_getEnergyConsumptionFromMeanPower(mi_getMeanPowerConsumption(p_nodeRow, crows), microseconds);
}


double mi_getStepBasedEnergyConsumption(row_struct ** p_nodeRow, int crows) {
	return mi_getStepBasedEnergyConsumptionInMilliJoule(p_nodeRow, crows) / 1000;
}

double mi_getStepBasedEnergyConsumptionInMilliJoule(row_struct ** p_nodeRow, int crows) {
	row_struct * nodeRow = *p_nodeRow;

	double energy = 0.0;

	if (crows >= 1) {

		// ignore first power consumption value, its not relevant for the desired timeframe
		uint64_t last_timestamp = nodeRow[0].timestamp;
		int j;
		for (j = 1; j < crows; ++j) {
			//watts * usecs = microjoule => divide by 1000 to get milliJoule
			energy += nodeRow[j].value * (nodeRow[j].timestamp - last_timestamp) / 1000;
			last_timestamp = nodeRow[j].timestamp;
		}
	}

	return energy;
}
