/*
 * DBConnector
 * http://eeclust.de
 *
 * Copyright (c) 2010-2012, University of Hamburg
 *
 * See the file COPYING in the package base directory for details
 *
 * main.c
 *
 *  Created on: Feb 22, 2011
 *  Author: Timo Minartz
 */

#include "DBConnector.h"
#include "MetricInterpreter.h"

int main(int argc, char** argv) {

	if (argc < 3) {
		printf("Invalid count of arguments.\n");
		printf("%s from_timestamp to_timestamp [node1] [node2] [...]\n",
				argv[0]);
		return 1;
	}

	uint64_t from_timestamp = atol(argv[1]);
	uint64_t to_timestamp = atol(argv[2]);

	if (strlen(argv[1]) == 19) // time in nanoseconds
		from_timestamp /= 1000.0;

	if (strlen(argv[2]) == 19) // time in nanoseconds
		to_timestamp /= 1000.0;

	if (strlen(argv[1]) != 19 && strlen(argv[1]) != 16) {
		printf("from_timestamp does not make sense.\n");
		return 2;
	}

	if (strlen(argv[2]) != 19 && strlen(argv[2]) != 16) {
		printf("to_timestamp does not make sense.\n");
		return 3;
	}

	int cnodes = argc - 3;

	double totalEnergy = 0.0;

	for (int i = 0; i < cnodes; ++i) {
		char * node = argv[i + 3];
		row_struct * rows;
		int crows = db_getNodeMetric("power", node, from_timestamp,
				to_timestamp, &rows);

		if (crows == 0) {
			printf("No measurement data for node %s for interval.\n", node);
			continue;
		}

		if (crows > 0) {

			double energy = mi_getStepBasedEnergyConsumption(&rows, crows);
			double power = mi_getStepBasedMeanPowerConsumption(&rows, crows);
			printf("%s - Duration: %f s Energy: %f J Mean Power: %f W\n", node,
					mi_getTimestampInSec(to_timestamp - from_timestamp),
					energy, power);
			totalEnergy += energy;
			free(rows);
		} else {
			printf("Query for node %s failed - Maybe non-existent node?\n",
					node);
		}
	}

	printf("ETS: %f J.\n", totalEnergy);

}

